/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.grapeshot.halfnes;

import com.grapeshot.halfnes.mappers.Mapper;
import java.util.Arrays;

/**
 *
 * @author Andrew
 */
public class CPURAM {
    private boolean hasprgram = true;
    private int[] wram = new int[2048];
    Mapper mapper;
    public APU apu;
    PPU ppu; //need these to call their write handlers from here.

    public CPURAM(final Mapper mappy) {
        mapper = mappy;
        // init memory
        Arrays.fill(wram, 0xff);
    }

    public int read(final int addr) {
//        if (addr > 65536) {
//            System.err.println("Read past end of RAM: "
//                    + utils.hex(addr));
//            //this is actually valid; the address just wraps.
//        }
        // converts cpu addrs to pointers to these arrays
        // System.err.println((addr & 0xffff) >> 12);
        //addr &= 0xffff;
        switch (((addr & 0xffff) >> 12)) {
            // switch on high 4bits of address

            case 0x0:
            case 0x1:
                // cpu ram: mirrored 4x
                return wram[addr & 0x7FF];
            case 0x2:
            case 0x3:
                // 8 byte ppu regs; mirrored lots
                return ppu.read(addr & 7);
            case 0x4:
                if (0x4000 <= addr && addr <= 0x4018) {
                    return apu.read(addr - 0x4000);
                }
            case 0x5:
            case 0x6:
            case 0x7:
            case 0x8:
            case 0x9:
            case 0xA:
            case 0xB:
            case 0xC:
            case 0xD:
            case 0xE:
            case 0xF:
                return mapper.cartRead(addr);
        }
        return (addr >> 8) & 0xff;

    }

    public void write(final int addr, final int data) {
//        if (data > 0xff) {
//            System.err.println("Invalid data, too big for a byte: "
//                    + utils.hex(data) + "@" + utils.hex(addr));
//        }
//        if (addr > 65536) {
//            System.err.println("Write past end of RAM"
//                    + utils.hex(addr));
//        }
        // converts cpu addrs to pointers to these arrays
        // System.err.println((addr & 0xffff) >> 12);
        switch ((addr >> 12) & 0xf) {
            // switch on high byte of address
            // allows easy bank switching with just change of a pointer
            // not rewriting massive hash table
            // trust me, i would LOVE to use pointer arithmetic but this is the
            // best i can do in java if i want 1k bank switch granularity
            case 0x0:
            case 0x1:
                // cpu ram: mirrored 4x
                wram[addr & 0x7FF] = (short) data;
                break;
            case 0x2:
            case 0x3:
                // 8 byte ppu regs; mirrored lots
                ppu.write(addr & 7, data);
                break;
            case 0x4:
                if (0x4000 <= addr && addr <= 0x4018) {
                    apu.write(addr - 0x4000, data);
                    break;
                }
            case 0x5:
            case 0x6:
            case 0x7:
            case 0x8:
            case 0x9:
            case 0xA:
            case 0xB:
            case 0xC:
            case 0xD:
            case 0xE:
            case 0xF:
                //all of this goes to the cartridge
                mapper.cartWrite(addr, data);
                break;
            default:
                System.err.println("Some read has gone terribly wrong.");
        }
    }

    public void write(final int addr, final int[] data) {
        for (int i = 0; i < data.length; ++i) {
            write((short) (addr + i), data[i]);
        }
    }

    public void setPrgRAMEnable(boolean b) {
        hasprgram = b;
    }



    public void setAPU(APU apu) {
        this.apu = apu;
    }

    public void setPPU(PPU ppu) {
        this.ppu = ppu;
    }
}
